function Locmodel=Buildlocal1a(Xtrn,Ytrn,Class,mod,MAXLV,MAXCV1SAMP,CVMOD,...
MINSAMP,Xcenter,Xcsamp);

%  This function computes and returns one new local PLS model. It is a 
%  subfunction of the function Iterate1a. 
%  I/O format is Locmodel=Buildlocal1a(Xtrn,Ytrn,Class,mod,MAXLV,MAXCV1SAMP,
%  CVMOD,MINSAMP,Xcenter,Xcsamp); Xtrn and Ytrn are the input and output 
%  training matrices repsectively. Class is the allocation of training samples
%  to different local regions and mod is the number of the local model 
%  (between 1 and MAXIT). The next four inputs are all transferred constants; 
%  MAXLV is the maximum number of latent variables, MAXCV1SAMP is the maximum 
%  number of samples when using full cross-validation, CVMOD is the number of 
%  cross-validation models, and MINSAMP is the minimum number of samples in a 
%  local region. Xcenter is the center of the local model, whereas Xcsamp is 
%  the number of the splitsample (between 1 and number_of_training_samples).
%
%  In the training data matrices it is assumed that samples are rows 
%  and variables are columns. The data are assumed scaled and centered.

%  Copyright
%  Lars Aarhus
%  1994

%If MLR = 1, use MLR instead of PLS as local modeling algorithm
MLR=0;

%Initialization
[trnobj,trnvar]=size(Xtrn);

%Find all the samples belonging to model mod
locnum=0;
for i=1:trnobj
  if Class(i)==mod
    Xloc=[Xloc; Xtrn(i,:)];
    Yloc=[Yloc; Ytrn(i,:)];
    locnum=locnum+1;
  end
end

if locnum < MINSAMP
  %Too few local samples
  Locmodel=0;
else
  %Scale to zero mean
  [Xlocmn,mnx]=mncn(Xloc);
  [Ylocmn,mny]=mncn(Yloc);
if MLR
  %===============================MLR==============================
  b=inv(Xlocmn'*Xlocmn)*Xlocmn'*Ylocmn;  
  minlv=trnvar;
else
  %===============================PLS================================
  %Define max latent variables in local model
  if locnum < MINSAMP*MAXLV
    maxlat=ceil(locnum/MINSAMP);
  else
    maxlat=MAXLV;
  end

  maxlat=min(MAXLV,locnum);

  %Compute a new model
  if locnum < MAXCV1SAMP
    [press,cumpress,minlv,b]=plscv1(Xlocmn,Ylocmn,maxlat,0);
  else
    [press,cumpress,minlv,b]=plscv(Xlocmn,Ylocmn,CVMOD,maxlat,0);
  end
end

  clc
  disp(['Number of samples in model ',num2str(mod),' is: ',num2str(locnum)]);
  pause(1);
  
  %Fixed centres
  Locmodel=[b; mny-mnx*b; Xcenter; Xcsamp; minlv];

  %Moving centres
  %Locmodel=[b; mny-mnx*b; mnx'; Xcsamp; minlv];
  end
end
