function[Newmodel,Class,Stop]=...
Iterate1a(Model,Xtrn,Xtrns,Ytrn,Class,K,Near,Pinv,Etype,Strategy,Cons,...
MAXLV,MAXCV1SAMP,CVMOD,MINSAMP);

%  In each iteration this function finds the splitsample and allocates the 
%  training samples, before starting computing the new local PLS model(s).
%  The function is a subfunction of the function Main1a.
%  I/0 format is [Newmodel,Class,Stop]=Iterate1a(Model,Xtrn,Xtrns,Ytrn,Class,K,
%  Near,Pinv,Etype,Strategy,Cons,MAXLV,MAXCV1SAMP,CVMOD,MINSAMP); Model is the
%  current total model before the iteration, whereas Xtrn and Ytrn are the 
%  input and output training matrices respectively, and Xtrns is the projected
%  training scores used during weighting of the local models. Class is the 
%  current allocation of training samples to different local regions, K is the
%  number of neighboring samples and Near is the nearest neighboring marix. 
%  Pinv is the inverse smoothing matrix of the projected training scores which
%  is computed once (in the function Main1a), and then only transferred to 
%  reduce computational demands. Etype is the type of error measure 
%  (1, 2 or 3), Strategy is the type of strategy for determining the 
%  splitsample (1 or 2), and Cons is a flag i indicating whether to include 
%  the consistency heuristic (Cons = 1) or not (Cons = 0). The final inputs 
%  are the transferred constants MAXLV, the maximum number of latent variables,
%  MAXCV1SAMP, the maximum number of samples hwne using full cross-validation, 
%  CVMOD, the number of cross-validation models, and MINSAMP, the minimum 
%  number of samples in a local region.
%  The outputs are Newmodel, the new total model after the iteration, Class, 
%  the new allocation of training samples and Stop, a flag indicating whether 
%  the algorithm can continue (Stop = 0), or has to be aborted because an 
%  error occured (Stop = 1 (No consistent training sample) or Step = 2 (Too 
%  few training samples in local model)).
%
%  In the training data matrices it is assumed that samples are rows 
%  and variables are columns. The data are assumed scaled and centered.

%  Copyright
%  Lars Aarhus
%  1994

%Initialization
[trnobj,A]=size(Xtrns);
[trnobj,xvar]=size(Xtrn);
[modpar,iter]=size(Model);

%If fixed centres
Xsplsamp=Model(xvar+A+2,:);

%Set Stop flag to false.
Stop=0;

%Compute the present error vector 
Etrn=zeros(size(Ytrn)); 
Ypred=Prediction1a(Model,Xtrn,Xtrns,Pinv); 
Etrn=Ytrn-Ypred;
plot(Ytrn,Ypred,'.');
xlabel('Y training')
ylabel('Y predicted')
title('Predicted vs "true" value of Y');
%save Direktor Ypred Ytrn 
pause(10);

%Strategy 1
if Strategy==1 
  Err=Etrn;
  Sample=1:trnobj;
else
%Strategy 2
  LEtrn=zeros(trnobj,iter);
  LSample=zeros(size(LEtrn));
  LNumber=zeros(1,iter);
  for obj=1:trnobj
    LNumber(Class(obj))=LNumber(Class(obj))+1;
    LEtrn(LNumber(Class(obj)),Class(obj))=Etrn(obj);
    LSample(LNumber(Class(obj)),Class(obj))=obj;
  end

%Find region with largest local RMSEE
  for i=1:iter
    LRMS(i)=RMS(LEtrn(1:LNumber(i),i));
  end
  [m,maxmodel]=max(LRMS);
  Err=LEtrn(1:LNumber(maxmodel),maxmodel);
  Sample=LSample(1:LNumber(maxmodel),maxmodel);
  disp('   ')
  disp(['Largest local RMSEE is ',num2str(m),' for model ',num2str(maxmodel)]);
end

disp('   ')
disp('Determining a new splitsample...')

if Etype==1
  %Find _the_ largest absolute error
  AEtrn=abs(Err);
  [maxerror,s]=max(AEtrn);
  splitnum=Sample(s);
  Split=Xtrns(splitnum,:);

  %Temporary save
  Tempsplit=Split;
  tempnum=splitnum;
 
  num=1;
  %Check consistency
  while ~(Consistent(splitnum,Near,Etrn,K,Cons) & ...
 ~Used(splitnum,Xsplsamp)) & num < length(Err)
    %'Remove' Split by setting its error to zero 
    AEtrn(s)=0;
    %Find next max splitsample.
    [maxerror,s]=max(AEtrn);
    splitnum=Sample(s);
    Split=Xtrns(splitnum,:);
    num=num+1;
  end

elseif Etype==2
  %Compute mean error for all the samples in Err
  MEtrn=zeros(size(Err));
  for obj=1:length(Err)  
    for k=1:K
      MEtrn(obj)=MEtrn(obj)+Etrn(Near(Sample(obj),k));  
    end;                                        
  end;
  MEtrn=abs(MEtrn)/K;
  [maxerror,s]=max(MEtrn);
  splitnum=Sample(s);
  Split=Xtrns(splitnum,:);

  %Temporary save
  Tempsplit=Split;
  tempnum=splitnum;
  
  num=1; 
  %Check consistency
  while ~(Consistent(splitnum,Near,Etrn,K,Cons) &...
 ~Used(splitnum,Xsplsamp)) & num < length(Err)
    %'Remove' Split by setting its error to zero 
    MEtrn(s)=0;
    %Find next max splitsample.
    [maxerror,s]=max(MEtrn);
    splitnum=Sample(s);
    Split=Xtrns(splitnum,:);
    num=num+1;
  end

else
  %Compute median error for all the samples in Err
  Metrn=zeros(length(Err),K);
  for obj=1:length(Err)
    for k=1:K
      Metrn(obj,k)=abs(Etrn(Near(Sample(obj),k)));  
    end
  end
  [maxerror,s]=max(median(Metrn'));
  splitnum=Sample(s);
  Split=Xtrns(splitnum,:);

  %Temporary save
  Tempsplit=Split;
  tempnum=splitnum;

  num=1;
  %Check consistency
  while ~(Consistent(splitnum,Near,Etrn,K,Cons) & ...
 ~Used(splitnum,Xsplsamp)) & num < length(Err)
    %'Remove' Split by setting its error to zero
    Metrn(s,:)=zeros(1,K);
    %Find next max splitsample.
    [maxerror,s]=max(median(Metrn'));
    splitnum=Sample(s);
    Split=Xtrns(splitnum,:);
    num=num+1;
  end
end

if num==length(Err)
%  Split=Tempsplit;
%  splitnum=tempnum;
  disp(['ERROR: No consistent samples found']);

  Stop=1; 
  Newmodel=Model;

else
  disp(['Done! Number ',num2str(splitnum),' (iteration ',num2str(num),')']);

  pause(3);

  %Reallocate all training samples i.e. compute new Class vector
  disp('   ')
  disp('Reallocating all the training samples...')
  Xcenter=[Model(xvar+2:xvar+A+1,:) Split'];
  Xsplsamp(iter+1)=splitnum;

  Class=zeros(trnobj,1);
  Validity=zeros(iter+1,1);
  for i=1:trnobj
    for j=1:iter+1
      Validity(j)=Weight(Xcenter(:,j),Xtrns(i,:)',Pinv);
    end
    [w,Class(i)]=max(Validity);
  end;

  disp('Done!');

  %Build new local models
  for mod=1:iter+1
    Local=Buildlocal1a(Xtrn,Ytrn,Class,mod,MAXLV,MAXCV1SAMP,CVMOD,MINSAMP,...
Xcenter(:,mod),Xsplsamp(mod));
    if Local==0
      disp('ERROR: Too few samples included in local model');
      Stop=2;
      pause(3);
    else
      Newmodel=[Newmodel Local];
    end
  end;
end;
