function[min,mod]=LocPLS1a(Xtrn,Ytrn,Xtst,Ytst,lambda,K,Etype,Strat,Cons)

%  This function projects the input training and test space onto a lower
%  dimensional space, spanned by the principal components of the training
%  space. The function is also the starting function for Algorithm 1a of the 
%  Local PLS Modeling algorithms. 
%  I/O format is [min,mod]=LocPLS1a(Xtrn,Ytrn,Xtst,Ytst,lambda,K,Etype,
%  Strat,Cons); Xtrn and Ytrn are the input and output training 
%  matrices respectively, whereas Xtst and Ytst are the input and output test 
%  matrices. lambda is the smoothing parameter, K is the number of neighboring
%  samples, Etype is the type of error measure (1, 2 or 3), Strat is the type 
%  of strategy for determining the splitsample (1 or 2), and Cons is a flag 
%  indicating whether to include the consistency heuristic (Cons = 1) or
%  not (Cons = 0). 
%  The outputs are min, the minimum RMSE and mod, the total prediction model 
%  at algorithm termination.
%
%  In the training and test data matrices it is assumed that samples are rows 
%  and variables are columns. The data are automatically scaled and centered.

%  Copyright
%  Lars Aarhus
%  1994

%Check the input dimensions 
[trnobj,trnvar]=size(Xtrn);
[tstobj,tstvar]=size(Xtst);

if trnvar==tstvar & trnobj >=5 & K <=trnobj

  %Autoscale the variables in training set to unit variance and zero mean
  [Xtrnaut,mxtrn,sxtrn]=auto(Xtrn);

  %Scale the variables in test set with the same factors
  Xtstaut=scale(Xtst,mxtrn,sxtrn);
  
  %Compute new principal component space
  [Xtrnscore,loads,ssq,res,q,tsq]=pca(Xtrnaut,0,0,3);
  [Xtstscore,resids,tsqs]=pcapro(Xtstaut,loads,ssq,q,tsq,0);
 
  %Start the main part of Algorithm 1a of the Local PLS Modeling algorithms  
  [min,mod]=...
Main1a(Xtrn,Xtrnscore,Ytrn,Xtst,Xtstscore,Ytst,lambda,K,Etype,Strat,Cons);
end
