function[RMSmin,Model]=...
Main1a(Xtrn,Xtrns,Ytrn,Xtst,Xtsts,Ytst,lambda,K,Etype,Strat,Cons)

%  This function controls the main iteration of Algorithm 1a of the Local
%  PLS Modeling algorithms. The function computes the nearest neighboring
%  matrix and the initial global PLS model, before starting the iteration.
%  I/O format is [RMSmin,Model]=Main1a(Xtrn,Xtrns,Ytrn,Xtst,Xtsts,Ytst,
%  lambda,K,Etype,Strat,Cons); Xtrn and Ytrn are the input and output training 
%  matrices respectively, whereas Xtst and Ytst are the input and output test 
%  matrices,. Xtrns and Xtsts are the projected training and test scores used
%  during weighting of the local models. lambda is the smoothing parameter, 
%  K is the number of neighboring samples, Etype is the type of error measure 
%  (1, 2 or 3), Strat is the type of strategy for determining the splitsample 
%  (1 or 2), and Cons is a flag indicating whether to include the consistency 
%  heuristic (Cons = 1) or not (Cons = 0). The outputs are RMSmin, the minimum
%  RMSE and Model, the complete prediction model at algorithm termination.
%
%  In the training and test data matrices it is assumed that samples are rows 
%  and variables are columns. The data are automatically scaled and centered.

%  Copyright
%  Lars Aarhus
%  1994

%If MLR = 1, use MLR instead of PLS as local modeling algorithm
MLR=0;

%Check the input dimensions
[trnobj,A]=size(Xtrns);
[trnobj,trnvar]=size(Xtrn);
[tstobj,tstvar]=size(Xtst);

%Autoscale the variables to unit variance and zero mean
[Xtrnaut,mxtrn,sxtrn]=auto(Xtrn);
[Ytrnaut,mytrn,sytrn]=auto(Ytrn); 

Xtstaut=scale(Xtst,mxtrn,sxtrn);
Ytstaut=scale(Ytst,mytrn,sytrn);

%Define the smoothing matrix
%P=diag(ones(A,1)*(lambda^2)); %Unit variance in all directions (point 1)
P=cov(Xtrns)*(lambda^2);     %(Co)variance matrix for training scores (point 2)
Pinv=inv(P);

%---Constants---
%Set the maximum number of latent variables
MAXLV=min(10,trnvar);

%Set the maximum number of samples when using full cross-validation (CV1)
MAXCV1SAMP=20;

%Set the number of cross-validation models (CV)
CVMOD=10;

%Set the minimum number of samples in a local region
MINSAMP=4;

%Set the maximum number of local models
MAXIT=15;

disp('Computing nearest neighboring matrix...');

%Compute the K nearest samples for all data points in the training set.
%Distance metric is Mahalanobis distance in the principal component space. 
Sinv=inv(cov(Xtrns));   %(Co)variance matrix for training scores
for obj=1:trnobj
  Xdist=Xtrns-ones(trnobj,1)*Xtrns(obj,:); 
  if trnvar==1
    %Can't use sum
    [s,order]=sort(((Xdist*Sinv).*Xdist)');
  else
    [s,order]=sort(sum(((Xdist*Sinv).*Xdist)'));    
  end
  Near= [Near; order(1:K)];
end
save Neighbour Near;
%Near=zeros(trnobj,trnobj);
%load NeighbourT10
disp('Done!');

%Compute initial global model. 
i=1;
if MLR
%=====================MLR=============================
b=inv(Xtrnaut'*Xtrnaut)*Xtrnaut'*Ytrnaut;   
minlv=trnvar;
else
%======================PLS========================
if trnobj < MAXCV1SAMP
  [press,cumpress,minlv,b] = plscv1(Xtrnaut,Ytrnaut,MAXLV,0);
else
  [press,cumpress,minlv,b]=plscv(Xtrnaut,Ytrnaut,CVMOD,MAXLV,0);
%  [press,cumpress,minlv,b]=plscvblk(Xtrnaut,Ytrnaut,CVMOD,MAXLV,0);
end;
end

Model = [b; 0; mean(Xtrns)'; 0; minlv]; %In fact unnecessary, centered already;

RMSmin = RMS(Ytstaut-Xtstaut*b);
RMStrain = RMS(Ytrnaut-Xtrnaut*b);
clc
disp(['RMSEP (test error) for initial PLS model: ',num2str(RMSmin)]);
disp(['RMSEE (training error) for initial model: ',num2str(RMStrain)]);
  
rmstesterror=RMSmin;
rmstrainerror=RMStrain;

Newmodel=Model;
rmsnew=RMSmin;

save LocMod Newmodel rmsnew    
save Modelerror rmstesterror rmstrainerror

plot(1:i,rmstesterror,'r+',1:i,rmstrainerror,'g*');
title('RMSEP (+) and RMSEE (*) for initial PLS model')
pause(3);

Class=ones(trnobj,1);

while i<MAXIT %rmsnew < RMSmin | i==1

  Model=Newmodel;
  i=i+1;
  [Newmodel,Class,Stop]=...
Iterate1a(Model,Xtrnaut,Xtrns,Ytrnaut,Class,K,Near,Pinv,Etype,Strat,Cons,...
MAXLV,MAXCV1SAMP,CVMOD,MINSAMP);
  
  if Stop
    i=MAXIT;
  else

  if trnvar==1
  plot(rescale(Xtrnaut,mxtrn,sxtrn),rescale(Ytrnaut,mytrn,sytrn),'x');
  hold
  plot(rescale(Xtstaut,mxtrn,sxtrn),rescale(Prediction1a(Newmodel,Xtstaut,Xtsts,Pinv),mytrn,sytrn));
  xlabel('X');
  ylabel('Y');
  title('Smooth local linear regression');
  hold off
  pause
  end

  RMSmin=rmsnew;

  rmsnew=RMS(Ytstaut-Prediction1a(Newmodel,Xtstaut,Xtsts,Pinv));
  RMStrain=RMS(Ytrnaut-Prediction1a(Newmodel,Xtrnaut,Xtrns,Pinv));
  clc
  disp(['RMSEP (test error) after ',num2str(i),' local PLS models: ',...
num2str(rmsnew)]);
  disp(['RMSEE (training error) after ',num2str(i),' local PLS models: ',...
num2str(RMStrain)]);

  rmstesterror=[rmstesterror rmsnew];
  rmstrainerror=[rmstrainerror RMStrain];

  plot(1:i,rmstesterror,'r--',1:i,rmstrainerror,'g-');
  xlabel('Number of local models')
  title(['Evolution of RMSEP (--) and RMSEE (-)'])
  pause(10);

  save LocMod Newmodel rmsnew    
  save Modelerror rmstesterror rmstrainerror
  end

end;


