function Ypred=Prediction1a(Model,Xtst,Xtsts,Pinv)

%  This function computes the output from one input sample, by using local PLS 
%  modeling. It is used during estimation and prediction.
%  I/O format is Ypred=Prediction1a(Model,Xtst,Xtsts,Pinv); where Model
%  is the total model, Xtst and Xtsts are the input sample and projected input
%  score respectively, and Pinv is the inverse smoothing matrix used during
%  weighting of the local models.
%  The output is Ypred, the predicted output value.

%  Copyright
%  Lars Aarhus
%  1994

%Initialization
[tstobj,A]=size(Xtsts);
[tstobj,xvar]=size(Xtst);
[modpar,mod]=size(Model);

B=Model(1:xvar,:);
B0=Model(xvar+1,:);
Xcenter=Model(xvar+2:xvar+A+1,:);

%Compute weighting matrix W
disp('   ')
disp('Computing weighting matrix...')
W=zeros(tstobj,mod);
for i=1:tstobj
  wsum=0;
  for j=1:mod
    w=Weight(Xcenter(:,j),Xtsts(i,:)',Pinv);
    wsum=wsum+w;
    W(i,j)=w;
  end
  W(i,:)=W(i,:)/wsum;
end

%save W;
disp('Done!')

%Ypred=(sum(((Xtst*B+ones(tstobj,1)*B0).*W)'))';
Ypred=((Xtst*B+ones(tstobj,1)*B0).*W)*ones(mod,1);



