function m = conpred(b,w,p,q,lv)
%  This function converts the matrices used for prediction
%  in PLS to a matrix of multiplication factors.  It works
%  with either single or multiple y-block variables.
%  If there is only 1 y-block variable each row of the output 
%  matrix corresponds to the contribution from each lv to
%  the y-block prediction.  If there are N y-block variables
%  each block of N rows corresponds to the contribution from
%  each lv to the prediction.
%  If there is only one y-block variable, you can use the 
%  cumsum function to get the sum over the rows, resulting in
%  a matrix of regression vectors where each row is the vector
%  for that number of lvs.  If there are multiple y-block 
%  variables, you can use conpred1 to get the matrix for any
%  specific number of lvs.
%  The I/O format is m = conpred(b,W,P,Q,lv)
%  where all the symbols have the usual meaning

% Copyright
% Barry M. Wise
% 1992
[r,c] = size(w);
[rq,cq] = size(q);
m = zeros(lv*rq,r);
m(1:rq,:) = [b(:,1)*w(:,1)*q(:,1)']';
if lv > 1
  temp = w(:,1)*p(:,1)';
end
for i = 1:lv-1
  if r > 100
    s = sprintf('Now converting Latent Variable number %g',i+1);
    disp(s);
  end
  wq = w(:,i+1)*q(:,i+1)';
  m(i*rq+1:(i+1)*rq,:) = [b(:,i+1)*(wq-temp*wq)]';
  if i ~= lv-1
    temp = temp + w(:,i+1)*p(:,i+1)' - temp*w(:,i+1)*p(:,i+1)';
  end
end
