% FUNCTION FTEST.M Function to evaluate the F distribution and its
% 	inverse using the incomplete beta function and its
%	inverse.  The function will return either the
%	probabilities given the F statistic and the degrees of
%	freedom or the F statistic given the probability and
%	degrees of freedom.  
% 
%	y = ftest(x,a,b,z) 
%
%	where 
%	   x = either the statistic (F) or the probability at which
%	       the function is to be evaluated.
%	   a = degrees of freedom
%	   b = degrees of freedom
%	   z = flag for determining how the function is evaluated
%	     = 1 percentage from F-test, given F ratio and D.F.'s
%	     = 2 F from inverse F-test given probability and D.F.'s
%
function y = ftest(x,a,b,z)
	aa = a * 0.5;
	if z == 1
		xx = a / (a + b * x);
		bb = b * 0.5;
		tmp = beta(aa,bb,xx);
		y = 1 - tmp;
	elseif z == 2
		bb = b * 0.50;
		xx = 1 - x;
		tmp = betai(xx,aa,bb);
		y = (a - a * tmp) / (b * tmp);
	else
		error('z must be 1 or 2')
		return
	end

