function [scores,res,tsqvals] = pcapro(newdata,loads,ssq,q,tsq,plots)
%  This function uses data from a previously calculated pca
%  model and projects new data onto the PCs and displays the
%  calculated limits.  The input variables are the new data,
%  the old loadings (loads), the old variance info (ssq), the limit
%  for q (q), the limit for t^2 (tsq) and an optional variable plots
%   which suppresses the plots when set to 0.  The outputs are the new
%   scores, residuals and t^2 values.  All of these are plotted as the
%  function proceeds if plots ~= 0.  The I/O format is:
%  [scores,resids,tsqs] = pcapro(newdata,loads,ssq,q,tsq,plots)
%  Warning:  Be sure to use the same scaling as original data!

%  Copyright
%  Barry M. Wise
%  1991, 1992

[m,n] = size(loads);
[ms,ns] = size(newdata);
[mss,nss] = size(ssq);
if nargin < 6
  plots = 1;
end
if ns ~= m
  error('Number of variables in new data not consistent with loadings')
end
scores = newdata*loads;
scl = 1:ms;
scllim = [1 ms];
temp = [1 1];
if plots ~= 0
  for i = 1:n
    pclim = temp*1.96*sqrt(ssq(i,2));
    plot(scl,scores(:,i),scllim,pclim,'--b',scllim,-pclim,'--b')
    xlabel('Sample Number')
    ylabel('Score')
    title('New Sample Scores with 95% Limits from Old Model')
    pause
  end
end
I = eye(m);
resmat = ((I - loads*loads')*newdata');
res = (sum(resmat.^2))'; 
lim = [q q];
if plots ~= 0
  plot(scl,res,scllim,lim)
  title('New Sample Residuals with Limits from Old Model')
  xlabel('Sample Number')
  ylabel('Residual')
  pause
end
temp2 = inv(diag(ssq(1:n,2)));
temp3 = scores*temp2;
tsqvals = sum((temp3.^2)');
if plots ~= 0
  tlim = [tsq tsq];
  plot(scl,tsqvals,scllim,tlim)
  title('New Samples Value of T^2 with 95% Limit From Old Model')
  xlabel('Sample Number')
  ylabel('Value of T^2')
end
