function [P,Q,W,T,U,b,ssqdif] = pls(x,y,lv)
%  This PLS program calculates the P, Q, W, T, and U matrices
%  for the given x and y blocks and the number of latent
%  variables specified.  It also outputs the fraction of
%  variance used in the x and y matrices.
%  Input format is as follows:
%  function [P,Q,W,T,U,b,ssqdif] = pls(x,y,lv);

%  Copyright
%  Barry M. Wise
%  1991

[mx,nx] = size(x);
[my,ny] = size(y);
if nx < lv
  error('No. of LVs must be <= no. of x-block variables')
end 
P = zeros(nx,lv);
Q = zeros(ny,lv);
W = zeros(nx,lv);
T = zeros(mx,lv);
U = zeros(my,lv);
b = zeros(1,lv);
ssq = zeros(lv,2);
ssqx = 0;
for i = 1:nx
  ssqx = ssqx + sum(x(:,i).^2);
end
ssqy = sum(sum(y.^2)');
for i = 1:lv
  [p,q,w,t,u] = plsnipal(x,y);
  b(1,i) = u'*t/(t'*t);
  x = x - t*p';
  y = y - b(1,i)*t*q';
  ssq(i,1) = (sum(sum(x.^2)'))*100/ssqx;
  ssq(i,2) = (sum(sum(y.^2)'))*100/ssqy;
  T(:,i) = t(:,1);
  U(:,i) = u(:,1);
  P(:,i) = p(:,1);
  W(:,i) = w(:,1);
  Q(:,i) = q(:,1);
end
ssqdif = zeros(lv,2);
ssqdif(1,1) = 100 - ssq(1,1);
ssqdif(1,2) = 100 - ssq(1,2);
for i = 2:lv
  for j = 1:2
    ssqdif(i,j) = -ssq(i,j) + ssq(i-1,j);
  end
end
disp('  ')
disp('        Percent Variance Captured by PLS Model')
disp('  ')
disp('             ----X-Block------   ----Y-Block------')
disp('      LV#    This LV    Total    This LV    Total ')
disp([(1:lv)' ssqdif(:,1) cumsum(ssqdif(:,1)) ssqdif(:,2) cumsum(ssqdif(:,2))])
