function [press,cumpress,minlv,b] = plscv(x,y,split,lv,noninter)
%  This function calculates the PRESS (press) and cumulative PRESS
%  (cumpress) for cross validating pls models.  It also specifies  
%  the number of latent variables at minimum PRESS (minlv)
%  and the regression vector (b) at minimum PRESS. 
%  It splits the data as specified by the user.  I/O format is: 
%  [press,cumpress,minlv,b] = plscv(x,y,split,lv,noninter);
%  where x is the independent block, y is the dependent block,
%  split is the number of fractions the data set is to be 
%  split into and lv is the maximum number of latent variables
%  to consider.  The last input, noninter, is optional and is
%  used to supress the prompt for user over-ride of the number
%  of latent variables chosen.  Any input here will suppress the prompt.

%  Copyright
%  Barry M. Wise
%  1991

[mx,nx] = size(x);
[my,ny] = size(y);
if mx ~= my
  error('Number of samples must be the same in both blocks')
end
press = zeros(split,lv);
clc
for i = 1:split
  home
  s = sprintf('Now working on test set %g out of %g',i,split);
  disp(s)
  ind = zeros(mx,1);
  count = 0;
  for j = 1:mx
    test = round((j+i-1)/split) - ((j+i-1)/split);
    if test == 0
      ind(j,1) = 1;
      count = count + 1;
    end
  end
  [a,b] = sort(ind);
  newx = x(b,:);
  newy = y(b,:);
  calx = newx(1:mx-count,:);
  testx = newx(mx-count+1:mx,:);
  caly = newy(1:mx-count,:);
  testy = newy(mx-count+1:mx,:);
  [P,Q,W,T,U,b,ssqdif] = pls(calx,caly,lv);
  for j = 1:lv
    ypred = plspred(testx,b,P,Q,W,j);
    press(i,j) = sum(sum((ypred-testy).^2));
  end
  plot(press(i,:))
  txt = sprintf('PRESS for Test Set Number %g out of %g',i,split);
  title(txt)
  xlabel('Number of Latent Variables')
  ylabel('PRESS')
end
pause(2)
cumpress = sum(press);
plot(cumpress)
title('Cumulative PRESS as a Function of Number of Latent Variables')
xlabel('Number of Latent Variables')
ylabel('PRESS')
[a,minlv] = min(cumpress);
t = sprintf('Minimum Cumulative PRESS is at %g LVs',minlv);
disp(t)
if nargin == 4
answ = input('Would you like to choose a different number of LVs?  (Yes = 1) ');
if answ == 1
  txt = sprintf('How many Latent Variables would you like? (Max = %g) ',minlv);
  minlv = input(txt);
end
end
disp('  ')
disp('Now working on final PLS model')
[P,Q,W,T,U,bb,ssqdif] = pls(x,y,minlv);
b = conpred1(bb,W,P,Q,minlv);
plot(b), hold on, plot(b,'o'), plot(zeros(b),'-g'), hold off
s = sprintf('Regression Coefficients in Final Model with %g LVs',minlv);
title(s)
xlabel('Variable Number')
ylabel('Regression Coefficient')
